#include <stdio.h>
#include "icode.h"
#include "lt.h"

void ICommand::print() const {
    char line[32];
    if (label != 0)
        printf("%s ", Label(label).toString(line, 32));
    switch (type) {
    case int_const:
        printf("int_const %d", int_value);
        break;
    case double_const:
        printf("double_const %lf", double_value);
        break;
    case string_const:
        printf("string_const \"%s\"", string_value);
        break;
    case assign:
        printf("assign");
        break;
    case name:
        printf("name %s", name_ptr->name);
        break;
    case name_lvalue:
        printf("name_lvalue %s", name_ptr->name);
        break;
    case def_variable:
        printf("def_variable %s of type ", name_ptr->name);

        LBaseType: ;
        switch(ext) {
        case TYPE_INT:
            printf("int"); break;
        case TYPE_DOUBLE:
            printf("double"); break;
        case TYPE_STRING:
            printf("string"); break;
        }
        break;
    case def_array:
        printf("def_array %s of type ", name_ptr->name);
        goto LBaseType;
        // break;
    case array_element:
        printf("array_element");
        break;
    case array_lvalue:
        printf("array_lvalue");
        break;
    case plus:
        printf("plus");
        break;
    case minus:
        printf("minus");
        break;
    case mul:
        printf("mul");
        break;
    case div:
        printf("div");
        break;
    case mod:
        printf("mod");
        break;
    case uminus:
        printf("uminus");
        break;
    case cmp:
        printf("cmp");
        break;
    case go_to:
        printf("goto %s", Label(goto_label).toString(line, 32));
        break;
    case if_goto:
        printf("if_goto ");
        switch (ext) {
        case eq:
            printf("eq ");
            break;
        case ne:
            printf("ne ");
            break;
        case gt:
            printf("gt ");
            break;
        case ge:
            printf("ge ");
            break;
        case lt:
            printf("lt ");
            break;
        case le:
            printf("le ");
            break;
        default:
            printf("illegal condition ");
        }
        printf("%s", Label(goto_label).toString(line, 32));
        break;
    case input:
        printf("input");
        break;
    case output:
        printf("output");
        break;
    case nop:
        printf("nop");
        break;
    default:
        printf("Illegal command");
    }
    printf("\n");
}

ICommand* IProgram::addNewCommand() {
    moveToEnd();
    ICommand* command = new ICommand();
    insertBefore(command);
    return command;
}

void IProgram::print() const {
    moveToBeg();
    while (!inEnd()) {
        ((ICommand*) pointerNext)->print();
        moveForward();
    }
}
