/*
 * L2-List of blocks allocated in dinamic memory
 */
#include <stdio.h>
#include "list.h"

void lst_link(block_ptr b1, block_ptr b2) {
    b1->next = b2;
    b2->prev = b1;
}

void lst_remove(block_ptr b) {  /* remove from list */
    lst_link(b->prev, b->next);
}

void lst_addAfter(block_ptr pointer, block_ptr new_block) {
    lst_link(new_block, pointer->next);
    lst_link(pointer, new_block);
}

void lst_addBefore(block_ptr pointer, block_ptr new_block) {
    lst_link(pointer->prev, new_block);
    lst_link(new_block, pointer);
}

/* Loop for every element.
   To break a loop, action(block_ptr b, void *arg1, void *arg2) must return TRUE.
   In this case, the function returns b;
   otherwise, it returns NULL */
block_ptr lst_loop(
    block_ptr header,   /* header of list, that is not in the list */
    int (*action)(block_ptr b, void *arg1, long arg2),
    void *arg1, long arg2
) {
    return lst_loopForward(header, header->next, action, arg1, arg2);
}

block_ptr lst_loopForward(
    block_ptr header,   /* header of list, that is not in the list */
    block_ptr start,    /* starting block */
    int (*action)(block_ptr b, void *arg1, long arg2),
    void *arg1, long arg2
) {
    block_ptr b = start;
    while (b != header) {
        if ((*action)(b, arg1, arg2) != 0) {
            return b;
        }
        b = b->next;
    }
    return NULL;
}

block_ptr lst_loopReverse(
    block_ptr header,       /* header of list, that is not in the list */
    block_ptr afterStart,   /* a block after a starting block */
    int (*action)(block_ptr b, void *arg1, long arg2),
    void *arg1, long arg2
) {
    block_ptr b = afterStart->prev;
    while (b != header) {
        if ((*action)(b, arg1, arg2) != 0) {
            return b;
        }
        b = b->prev;
    }
    return NULL;
}
