#ifndef NAMETAB_H
#define NAMETAB_H

#include "list.h"
#include "flagbits.h"

/* Name table is L2-list of name entries */
typedef block_header name_table, *name_table_ptr;

#define MAXNAME 127     /* Maximal length of name */

/* Base types */
#define BASETYPE_INT  0
#define BASETYPE_CHAR 1
#define BASETYPE_VOID 2

/* Declarators: simple variable, pointer, array, function */
#define DECL_VAR 0
#define DECL_PNT 1
#define DECL_ARR 4
#define DECL_FUN 8

/* Definition of type */
typedef struct type_tag {
    int base_type;      /* BASETYPE_INT, BASETYPE_CHAR, BASETYPE_VOID */
    int decl;           /* DECL_VAR, DECL_PNT, DECL_ARR, DECL_FUN */
    int base_type_size;         /* size of single element of base type */
    int element_size;           /* size of single element */
    int dimension;              /* dimension of array in type units */
    int size;                   /* size in bytes */
    int num_arguments;          /* number of arguments */
    name_table arguments;       /* list of function arguments */
} type, *type_ptr;

/* Element of name table */
typedef struct name_entry_tag {
    block_header link;          /* link to next / prev */
    char name[MAXNAME + 1];     /* name */
    type typ;                   /* type of variable */
    int offset;                 /* offset from frame pointer */
    int arg_number;             /* 1-based argument number for arguments */
    int modifiers;              /* various flags */
} name_entry, *name_ptr;

/* Bits in the modifiers field of struct name_entry_tag
   (defined in "flagbits.h"):
MOD_LOCAL               This is local variable
MOD_EXTERN              extern (definition in another file)
MOD_FUNCT_BODY_DEFINED  Function definition is given
*/

void copy_type(type_ptr dst, type_ptr src);
void copy_name_entry(name_ptr dst, name_ptr src);
void remove_name_entry(name_ptr n);     /* remove from list, release */
void add_name_entry(name_table_ptr t, name_ptr n); /* add to list */
void release_name_table(name_table_ptr t); /* release all elements */
name_ptr name_lookup(const char *name, name_table_ptr t);

void initialize_type(type_ptr t);
void erase_type(type_ptr t);
name_ptr allocate_name_entry();
void release_name_entry(name_ptr);

void print_name_table(name_table_ptr t);

/* Global variables */
extern name_table global_table, local_table;
extern name_table_ptr current_table;
extern void init_name_tables();

#endif /* NAMETAB_H */
