//
// Name table
//
#ifndef NT_H
#define NT_H
#include <string.h>
#include "lextypes.h"
#include "lst.h"
#include "txtstr.h"
#include "value.h"

class NameDef: public L1ListHeader {
public:
    const char* name;
    int baseType;
    int dimension;
    int numDefinitions;
    int numReads;
    int numWrites;
    int modifiers;
    // Bits in modifiers
    enum {
        NAME_VARIABLE = 1,
        NAME_ARRAY = 2,
        NAME_FUNCTION = 4
    };
    Value value; // value of simple variable
    void* array; // array of values for array variable

    NameDef():
        name(0),
        baseType(TYPE_UNDEFINED),
        dimension(0),
        numDefinitions(0),
        numReads(0),
        numWrites(0),
        modifiers(0),
        value(),
        array(0)
    {}

    NameDef(const NameDef& n):
        name(n.name),
        baseType(n.baseType),
        dimension(n.dimension),
        numDefinitions(n.numDefinitions),
        numReads(n.numReads),
        numWrites(n.numWrites),
        modifiers(n.modifiers),
        value(n.value),
        array(0)
    {}

    NameDef(const char* str):
        name(str),
        baseType(TYPE_UNDEFINED),
        dimension(0),
        numDefinitions(0),
        numReads(0),
        numWrites(0),
        modifiers(0),
        value(),
        array(0)
    {}

    NameDef& operator=(const NameDef& n) {
        name = n.name;
        baseType = n.baseType;
        dimension = n.dimension;
        numDefinitions = n.numDefinitions;
        numReads = n.numReads;
        numWrites = n.numWrites;
        modifiers = n.modifiers;
        value = n.value;
        array = 0;
        return *this;
    }

    bool operator==(const NameDef& n) const {
        return (strcmp(name, n.name) == 0);
    }

    bool operator==(const char* str) const {
        return (strcmp(name, str) == 0);
    }

    bool operator!=(const NameDef& n) const {
        return (!(*this == n));
    }

    bool operator!=(const char* str) const {
        return (!(*this == str));
    }

    operator const char*() const {
        return name;
    }

    virtual ~NameDef() {
        if (array != 0) {
            switch (baseType) {
            case TYPE_INT:
                delete[] ((int*) array); break;
            case TYPE_DOUBLE:
                delete[] ((double*) array); break;
            case TYPE_STRING:
                delete[] ((TextString*) array); break;
            default:
                assert(false); // Illegal type of array
            }
        }
    }

    void allocateArray(int n);
};

//
// Name table
//
class NameTable: public L1List {
public:
    NameDef* findName(const char *name) const;
    NameDef* addName(const char *name);
};

#endif
