%{
/* Scanner tor SimpComp */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "scanner.h"

char strbuf[STRMAX+1];
static char *strptr, strend;
static char *stradd(char *d, char *s, int n);

extern YYSTYPE yylval;

void errtxt(const char *txt);
void errchar(int c);
%}

%option yylineno

%Start   S COM STR

LETTER    [A-Z_a-z]
NOLETTER [^A-Z_a-z]
DIGIT    [0-9]
IDENT    {LETTER}({LETTER}|{DIGIT})*
NUMBER   {DIGIT}+
WHISP1   [ \t\r]
WHISP    ({WHISP1}|\n)

%%  
%{
                char c, *p;
%}
                { BEGIN S; }

<S>"int"        { yylval.int_value = INT;  return( TYPE ); }
<S>"double"     { yylval.int_value = DOUBLE; return( TYPE ); }
<S>"while"      { return( WHILE ); }
<S>"endwhile"   { return( ENDWHILE ); }
<S>"if"         { return( IF ); }
<S>"endif"      { return( ENDIF ); }
<S>"else"       { return( ELSE ); }
<S>"elseif"     { return( ELSEIF ); }
<S>"break"      { return( BREAK ); }
<S>"continue"   { return( CONTINUE ); }
<S>"return"     { return( RETURN ); }

<S>{IDENT}      { strncpy(yylval.text_value, yytext, NAMEMAX);
                  return( NAME ); }
<S>{NUMBER}     { yylval.int_value = atoi(yytext); return( ICON ); }
<S>"+"          { yylval.int_value = PLUS; return( PLUS ); }
<S>"-"          { yylval.int_value = MINUS; return( MINUS ); }
<S>"*"          { yylval.int_value = MUL; return( MUL ); }
<S>"&&"         { yylval.int_value = LAND; return( LAND ); }
<S>"||"         { yylval.int_value = LOR; return( LOR ); }
<S>"!"          { yylval.int_value = LNOT; return( LNOT ); }
<S>"="          { yylval.int_value = ASG; return( ASG ); }
<S>"<"          { yylval.int_value = LT; return( RELOP ); }
<S>"<="         { yylval.int_value = LE; return( RELOP ); }
<S>">"          { yylval.int_value = GT; return( RELOP ); }
<S>">="         { yylval.int_value = GE; return( RELOP ); }
<S>"=="         { yylval.int_value = EQ; return( EQUOP ); }
<S>"!="         { yylval.int_value = NE; return( EQUOP ); }
<S>"/"          { yylval.int_value = DIV; return( DIVOP ); }
<S>"%"          { yylval.int_value = MOD; return( DIVOP ); }
<S>"<<"         { yylval.int_value = LS; return( SHIFTOP ); }
<S>">>"         { yylval.int_value = RS; return( SHIFTOP ); }
<S>"++"         { yylval.int_value = ICR;  return( ICOP ); }
<S>"--"         { yylval.int_value = DECR; return( ICOP ); }
<S>"("          { return( LP ); }
<S>")"          { return( RP ); }
<S>"["          { return( LB ); }
<S>"]"          { return( RB ); }
<S>";"          { return( SM ); }
<S>","          { return( CM ); }
<S>":"          { return( COLON ); }

<S>"/*"         { BEGIN COM; }
<COM>[^*]*      { }
<COM>"*"/[^/]   { }
<COM>"*/"       { BEGIN S; }

<S>\'           { strend='\''; strptr=strbuf; BEGIN STR; }
<S>\"           { strend='\"'; strptr=strbuf; BEGIN STR; }
<STR>[^'"\\\n]+ { if (strptr+yyleng <= strbuf+STRMAX)
                      strptr = stradd(strptr, yytext, yyleng);
                  else { 
                      strtoolong: strptr=strbuf;
                      errtxt( "String is too long" );
                  }
                }
<STR>\\n        { c = '\n'; goto addchar; }
<STR>\\t        { c = '\t'; goto addchar; }
<STR>\\r        { c = '\r'; goto addchar; }
<STR>\\b        { c = '\b'; goto addchar; }
<STR>\\f        { c = '\f'; addchar:
                  if (strptr<strbuf+STRMAX) *strptr++ = c;
                  else goto strtoolong; }
<STR>\\([0-7]{1,3}) { for( p=yytext+1,c=0; *p; )
                      c = (c<<3) + *p++ -'0';
                    goto addchar; }
<STR>\\.        { c = yytext[1]; goto addchar; }
<STR>\\\n       { }
<STR>['"]       { if( yytext[yyleng-1] == strend ) {
                   stroff: *strptr=0; BEGIN S;
                   if( strend == '\'' ) {
                     if( strptr != strbuf+1 )
                      errtxt("Illegal string");
                     yylval.int_value = *strbuf;
                     return( ICON );
                   } else {
                     /* String is passed in strbuf */
                     return( STRING );
                   }
                  } else { c=yytext[0]; goto addchar; }
                }
<STR>\n         { errtxt("Unterminated string"); goto stroff; }

<S>{WHISP}+     { }
<S>.            { errchar(yytext[0]); }

%%

static char *stradd(char *d, char *s, int n) {
    for( ; n; n-- ) *d++ = *s++;
    return( d );
}

int yywrap() { return( 1 ); }

void errtxt(const char *txt) {
    printf("Scan error: %s\n", txt);
}

void errchar(int c) {
    printf("Scan error: illegal character 0x%02x\n", (c & 0xff));
}
